#
# daemon.py - SAKU daemon module.
# Copyright (c) 2005 shinGETsu Project.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: daemon.py,v 1.9 2005/10/17 23:55:28 fuktommy Exp $
#

import os
import sys
import time

import config
import httpd
import crond
from upnp import findrouter

__version__ = "$Revision: 1.9 $"

router = None

class Logger:

    """Save logs to /LOGDIR/%Y-%m-%d."""

    def __init__(self, logdir):
        self.logdir = logdir
        self.logfile = ""
        self.output = None

    def write(self, msg):
        newlog = self.logdir + "/" + \
                 time.strftime("%Y-%m-%d", time.localtime())
        if self.logfile == newlog:
            pass
        elif self.logfile == "":
            self.logfile = newlog
            self.output = file(self.logfile, "a")
        else:
            self.output.close()
            self.logfile = newlog
            self.output = file(self.logfile, "a")
        self.output.write(msg)
        self.output.flush()

    def flush(self):
        pass

    def close(self):
        pass

def setup():
    config.flags.append("light_cgi")
    for i in [os.path.join(config.docroot, j) for j in \
                (config.run_dir, config.cache_dir)] + \
             [config.log_dir]:
        if not os.path.isdir(i):
            os.makedirs(i)

def set_logger():
    logger = Logger(os.path.join(os.getcwd(), config.log_dir))
    sys.stderr = logger
    sys.stdout = logger

def start_daemon():
    global router
    if config.use_upnp:
        sys.stderr.write("finding router\n")
        router = findrouter(config.upnp_timeout)
        if router:
            sys.stderr.write("found router: %s\n" % router)
        else:
            sys.stderr.write("Error: faild to find router\n")

    crondaemon = crond.Crond(router)
    crondaemon.setDaemon(True)
    crondaemon.start()

    httpdaemon = httpd.Httpd()
    httpdaemon.setDaemon(True)
    httpdaemon.start()

    return httpdaemon

def stop_daemon():
    if router:
        sys.stderr.write("sending router closeport %d: %s\n" %
                         (config.port, router))
        flag = router.closeport(config.port, "TCP")
        if flag:
            sys.stderr.write("closeport succeed: %s\n" % router)
        else:
            sys.stderr.write("Error: closeport failed: %s\n" % router)
